from jbag.image import windowing


def cavass_soft_tissue_windowing(input_data):
    return windowing(input_data, 1000, 500)


def cavass_bone_windowing(input_data):
    return windowing(input_data, 2000, 4000)


def cavass_pet_windowing(input_data):
    return windowing(input_data, 1200, 3500, invert=True)



import os
import shutil
from uuid import uuid4

import numpy as np
from jbag.io import save_nifti, ensure_output_file_dir_existence
from jbag.medical_image_converters import nifti2dicom

from cavass.ops import execute_cmd, get_voxel_spacing, read_cavass_file
from cavass.utils import one_hot


def dicom2cavass(input_dir, output_file, offset_value=0):
    """
    Note that if the output file path is too long, this command may be failed.

    Args:
        input_dir (str or pathlib.Path):
        output_file (str or pathlib.Path):
        offset_value (int, optional, default=0):

    """

    assert os.path.exists(input_dir), f'{input_dir} does not exist!'

    made_output_dir, output_dir = ensure_output_file_dir_existence(output_file)

    try:
        r = execute_cmd(f'from_dicom {input_dir}/* {output_file} +{offset_value}')
    except Exception as e:
        if made_output_dir and os.path.isdir(output_dir):
            shutil.rmtree(output_dir)
        else:
            if os.path.exists(output_dir):
                os.remove(output_file)
        raise e

    return r


def nifti2cavass(input_file, output_file, offset_value=0, dicom_accession_number=1):
    """
    Convert NIfTI image to cavass image.

    Args:
        input_file (str or pathlib.Path):
        output_file (str or pathlib.Path):
        offset_value (int, optional, default=0):
        dicom_accession_number (int, optional, default=1):
    """

    assert os.path.isfile(input_file), f'{input_file} does not exist or is not a file!'

    made_output_dir, output_dir = ensure_output_file_dir_existence(output_file)

    tmp_dicom_dir = os.path.join(output_dir, f'{uuid4()}')
    try:
        r1 = nifti2dicom(input_file, tmp_dicom_dir, dicom_accession_number)
        r2 = dicom2cavass(tmp_dicom_dir, output_file, offset_value)
    except Exception as e:
        if made_output_dir and os.path.isdir(output_dir):
            shutil.rmtree(output_dir)
        if os.path.isdir(tmp_dicom_dir):
            shutil.rmtree(tmp_dicom_dir)
        if os.path.exists(output_file):
            os.remove(output_file)

        raise e
    shutil.rmtree(tmp_dicom_dir)
    return r1, r2


def cavass2nifti(input_file, output_file, orientation='ARI'):
    """
    Convert cavass IM0 and BIM formats to NIfTI.

    Args:
        input_file (str or pathlib.Path):
        output_file (str or pathlib.Path):
        orientation (str, optional, default="ARI"): Image orientation of NIfTI file, `ARI` or 'LPI'

    Returns:

    """

    assert os.path.isfile(input_file), f'{input_file} does not exist or is not a file!'

    spacing = get_voxel_spacing(input_file)
    data = read_cavass_file(input_file)
    save_nifti(output_file, data, spacing, orientation=orientation)


def nifti_label2cavass(input_file, output_file, objects, discard_background=True):
    """
    Convert NIfTI format segmentation file to cavass BIM format file. A NIfTI file in where contains arbitrary categories
    of objects will convert to multiple CAVASS BIM files, which matches to the number of object categories.

    Args:
        input_file (str or pathlib.Path):
        output_file (str): The final saved file for category i in input segmentation will be
        `output_file_prefix_{objects[i]}.BIM`
        objects (sequence or str): Objects is an array or a string with comma splitter of object categories,
        where the index of the category in the array is the number that indicates the category in the segmentation.
        discard_background (bool, optional, default True): If True, the regions with label of 0 in the segmentation
        (typically refer to the background region) will not be saved.

    Returns:

    """
    import nibabel as nib

    assert os.path.isfile(input_file), f'{input_file} does not exist or is not a file!'

    input_data = nib.load(input_file)
    image_data = input_data.get_fdata()

    if isinstance(objects, str):
        objects = objects.split(',')
    one_hot_arr = one_hot(image_data, num_classes=len(objects))

    start = 1 if discard_background else 0
    for i in range(start, one_hot_arr.shape[3]):
        nifti_label_image = nib.Nifti1Image(one_hot_arr[..., i], input_data.affine, input_data.header, dtype=np.uint8)
        tmp_nifti_file = f'{output_file}_{objects[i]}.nii.gz'
        made_output_dir, output_dir = ensure_output_file_dir_existence(tmp_nifti_file)
        try:
            nib.save(nifti_label_image, tmp_nifti_file)
            nifti2cavass(tmp_nifti_file, f'{output_file}_{objects[i]}.BIM')
        except Exception as e:
            if made_output_dir and os.path.isdir(output_dir):
                shutil.rmtree(output_dir)
            if os.path.exists(tmp_nifti_file):
                os.remove(tmp_nifti_file)
            raise e
        os.remove(tmp_nifti_file)



import os.path
import shutil
import uuid
from typing import Union, LiteralString, Sequence

from jbag.io import ensure_output_file_dir_existence

from cavass.constants import CAVASS_START_INDEX
from cavass.ops import ndvoi, matched_reslice, bin_ops
from cavass.slice_range import get_slice_range


def integrate(untrimmed_file: Union[str, LiteralString],
              trimmed_files: Sequence[Union[str, LiteralString]],
              output_file_1: Union[str, LiteralString],
              output_file_2: Union[str, LiteralString]):
    """
    Suture trimmed CAVASS files.
    For the situation that the whole file is trimmed into multiple files of body regions, this script integrates
    multiple trimmed files into one file.
    The ROI of the new file is determined by the provided trimmed files, which is the region exactly contains all
    trimmed files and may not be the same as the ROI of original untrimmed file.

    Args:
        untrimmed_file (str or LiteralString): The original untrimmed file.
        trimmed_files (Sequence): Trimmed files. trimmed files must be arranged correctly in the order of form inferior to superior.
        output_file_1 (str or LiteralString): `output_file_1` is the output file trimmed from the untrimmed file according to the ROI obtained from the untrimmed files.`
        output_file_2 (str or LiteralString): `output_file_2` is the output file of integrated files of the untrimmed files.

    Returns:

    """

    assert os.path.exists(untrimmed_file), f'{untrimmed_file} does not exist!.'
    for each in trimmed_files:
        assert os.path.exists(each), f'{each} does not exist!.'

    # Get ROI from the first and last untrimmed files.
    # The inferior location is the inferior location of the first trimmed image in the untrimmed image.
    # The superior location is the superior location of the last trimmed image in the untrimmed image.
    all_unmatched_files = []
    inferior_slice_idx, _, unmatched_files = get_slice_range(trimmed_files[0], untrimmed_file)
    all_unmatched_files.extend(unmatched_files)
    _, superior_slice_idx, unmatched_files = get_slice_range(trimmed_files[-1], untrimmed_file)
    all_unmatched_files.extend(unmatched_files)

    # CAVASS uses index started from 1.
    inferior_slice_idx -= CAVASS_START_INDEX
    superior_slice_idx -= CAVASS_START_INDEX

    # The new ROI is [inferior_slice_idx, superior_slice_idx]
    # Trim from original untrimmed file.

    made_output_dir_1, output_dir_1 = ensure_output_file_dir_existence(output_file_1)

    try:
        ndvoi(untrimmed_file, output_file_1, min_slice_dim_3=inferior_slice_idx, max_slice_dim_3=superior_slice_idx)
    except Exception as e:
        if made_output_dir_1 and os.path.isdir(output_dir_1):
            shutil.rmtree(output_dir_1)
        if os.path.exists(output_file_1):
            os.remove(output_file_1)
        raise e

    made_output_dir_2, output_dir_2 = ensure_output_file_dir_existence(output_file_2)

    # create reslice files of trimmed files w.r.t untrimmed file.
    tmp_files = []
    reslice_files = []
    file_type = os.path.splitext(trimmed_files[0])[1][1:]
    interpolation_method = 'nearest' if file_type == 'BIM' else 'linear'
    for trimmed_file in trimmed_files:
        reslice_file = os.path.join(output_dir_2, f'{uuid.uuid1()}.{file_type}')
        tmp_files.append(reslice_file)
        reslice_files.append(reslice_file)
        try:
            matched_reslice(trimmed_file, untrimmed_file, reslice_file, interpolation_method=interpolation_method)
        except Exception as e:
            if made_output_dir_1 and os.path.isdir(output_dir_1):
                shutil.rmtree(output_dir_1)
            if os.path.exists(output_file_1):
                os.remove(output_file_1)

            if made_output_dir_2 and os.path.isdir(output_dir_2):
                shutil.rmtree(output_dir_2)
            for each in tmp_files:
                if os.path.exists(each):
                    os.remove(each)
            raise e

    # integrate reslice files by OR operation.
    if len(reslice_files) > 1:
        or_op_file_1 = reslice_files[0]
        or_op_file_2 = reslice_files[1]
        or_op_output_file = os.path.join(output_dir_2, f'{uuid.uuid1()}.{file_type}')
        or_op_output_files = [or_op_output_file]
        tmp_files.append(or_op_output_file)
        bin_ops(or_op_file_1, or_op_file_2, or_op_output_file, op='or')
        if len(reslice_files) > 2:
            for reslice_file in reslice_files[2:]:
                or_op_file_1 = or_op_output_files[-1]
                or_op_file_2 = reslice_file
                or_op_output_file = os.path.join(output_dir_2, f'{uuid.uuid1()}.{file_type}')
                or_op_output_files.append(or_op_output_file)
                tmp_files.append(or_op_output_file)
                try:
                    bin_ops(or_op_file_1, or_op_file_2, or_op_output_file, op='or')
                except Exception as e:
                    if made_output_dir_1 and os.path.isdir(output_dir_1):
                        shutil.rmtree(output_dir_1)
                    if os.path.exists(output_file_1):
                        os.remove(output_file_1)

                    if made_output_dir_2 and os.path.isdir(output_dir_2):
                        shutil.rmtree(output_dir_2)
                    for each in tmp_files:
                        if os.path.exists(each):
                            os.remove(each)
                    raise e

        output_file = or_op_output_files[-1]
    else:
        output_file = reslice_files[0]

    # reslice to otuput file 1
    try:
        matched_reslice(output_file, output_file_1, output_file_2, interpolation_method=interpolation_method)
    except Exception as e:
        if made_output_dir_1 and os.path.isdir(output_dir_1):
            shutil.rmtree(output_dir_1)
        if os.path.exists(output_file_1):
            os.remove(output_file_1)

        if made_output_dir_2 and os.path.isdir(output_dir_2):
            shutil.rmtree(output_dir_2)
        for each in tmp_files:
            if os.path.exists(each):
                os.remove(each)
        raise e

    for each in tmp_files:
        if os.path.exists(each):
            os.remove(each)
    return all_unmatched_files



import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='cavass',
    version='1.1.3',
    author='Dai Jian',
    author_email='daijian@stumail.ysu.edu.cn',
    description='CAVASS python APIs.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=setuptools.find_packages(),
    classifiers=[
        'Programming Language :: Python :: 3.11',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
    ],
    python_requires='~=3.11',
    install_requires=[
        'jbag >= 4.0.0',
    ]
)
